/*
 * encryption.c:
 * Packet encryption/decryption for vpn-udp.
 *
 * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
 * Email: chris@ex-parrot.com; WWW: http://www.ex-parrot.com/~chris/
 *
 */

static const char rcsid[] = "$Id: encryption.c,v 1.1.1.1 2001/08/30 18:24:39 chris Exp $";

#include <sys/types.h>

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>

#include <openssl/blowfish.h>

#include "vpn-udp.h"

extern int verbose; /* in main.c */

/* setup_blowfish:
 * Set up Blowfish key, ready for encryption. */
BF_KEY bfkey;
unsigned char *bf_pkt;

int setup_encryption(char *file) {
    int fd = open(file, O_RDONLY);
    ssize_t n;
    unsigned char key[16];
    if (fd == -1) {
        print_log(LOG_ERR, "open(`%s'): %s", file, strerror(errno));
        return 0;
    }
    n = read(fd, key, 16);
    if (n == -1) {
        print_log(LOG_ERR, "read(`%s'): %s", file, strerror(errno));
        return 0;
    } else if (n < 16) {
        print_log(LOG_ERR, "key file `%s' does not contain 16 bytes of data", file);
        return 0;
    }

    BF_set_key(&bfkey, 16, key);

    bf_pkt = (unsigned char*)malloc(BUFLEN);

    if (verbose)
        print_log(LOG_ERR, "loaded 16-byte Blowfish key from `%s'", file);

    return 1;
}

/* encrypt_packet:
 * Encrypt a packet. */
void encrypt_packet(unsigned char *packet, int length) {
    char ivec[16] = {0};
    BF_cbc_encrypt(packet, bf_pkt, length, &bfkey, ivec, BF_ENCRYPT);
    memcpy(packet, bf_pkt, length);
}

/* decrypt_packet:
 * Decrypt a packet. */
void decrypt_packet(unsigned char *packet, int length) {
    char ivec[16] = {0};
    BF_cbc_encrypt(packet, bf_pkt, length, &bfkey, ivec, BF_DECRYPT);
    memcpy(packet, bf_pkt, length);
}

